package SPVM::Regex::Pattern;

1;

=head1 NAME

SPVM::Regex::Pattern - Regex::Pattern in SPVM | Regular expression pattern

=head1 SYNOPSYS
  
  use Regex::Pattern;

=head1 DESCRIPTION

L<Regex::Pattern|SPVM::Regex::Pattern> is a internal data structure which represents regular expression patterns for L<Regex|SPVM::Regex>.
