package SPVM::Hash::Entry;

1;

=head1 NAME

SPVM::Hash::Entry - Hash::Entry in SPVM | Hash entry

=head1 SYNOPSYS
  
  use Hash::Entry;

=head1 DESCRIPTION

L<Hash::Entry|SPVM::Hash::Entry> is used for L<Hash|SPVM::Hash> internally.
