package SPVM::BlessedObject;

1;

=head1 NAME

SPVM::BlessedObject - Blessed object base class

=head1 DESCRIPTION

SPVM::BlessedObject is blessed object base class.

Current subclasses is L<SPVM::BlessedObject::Array> and L<SPVM::BlessedObject::Class>.
